@@ -29,7 +29,7 @@ fill_pipeline_and_check (GstElement * comp, GList * segments, GList * seeks)
   CollectStructure *collect;
   GstBus *bus;
   GstMessage *message;
-  gboolean carry_on = TRUE;
+  gboolean carry_on = TRUE, expected_failure;
   GstPad *sinkpad;
   GList *ltofree = seeks;
 
@@ -46,6 +46,7 @@ fill_pipeline_and_check (GstElement * comp, GList * segments, GList * seeks)
 
   /* Expected segments */
   collect->expected_segments = segments;
+  collect->keep_expected_segments = TRUE;
 
   g_signal_connect (G_OBJECT (comp), "pad-added",
       G_CALLBACK (composition_pad_added_cb), collect);
@@ -86,14 +87,16 @@ fill_pipeline_and_check (GstElement * comp, GList * segments, GList * seeks)
 
           if (seeks == NULL) {
             carry_on = FALSE;
+            g_list_free_full (collect->expected_segments, g_free);
+            collect->expected_segments = NULL;
+            GST_DEBUG ("Done seeking");
             break;
           }
 
-          /* We should have received the segment by then and there should be none left */
-          fail_if (collect->expected_segments != NULL,
-              "Didn't receive segment corresponding to seek");
-
-          while (seeks) {
+          g_list_free_full (collect->expected_segments, g_free);
+          collect->expected_segments = NULL;
+          expected_failure = TRUE;
+          while (expected_failure && carry_on) {
             SeekInfo *sinfo = (SeekInfo *) seeks->data;
 
             seeks = seeks->next;
@@ -105,9 +108,10 @@ fill_pipeline_and_check (GstElement * comp, GList * segments, GList * seeks)
                   g_list_append (collect->expected_segments, segment_new (1.0,
                       GST_FORMAT_TIME, sinfo->start, sinfo->stop,
                       sinfo->position));
+
+              expected_failure = FALSE;
             }
 
-            /* Seek to 0.5s */
             GST_DEBUG ("Seeking to %" GST_TIME_FORMAT ", Expecting (%"
                 GST_TIME_FORMAT " %" GST_TIME_FORMAT ")",
                 GST_TIME_ARGS (sinfo->position), GST_TIME_ARGS (sinfo->start),
@@ -122,7 +126,6 @@ fill_pipeline_and_check (GstElement * comp, GList * segments, GList * seeks)
               break;
             }
 
-            GST_DEBUG ("Seek failed as expected");
             if (seeks == NULL)
               carry_on = FALSE;
             g_free (sinfo);
